@@ -44,10 +44,15 @@ from scipy.spatial import cKDTree
 from ..util.recipes import *
 from ..util.constants import _get_grav
 from ..util.plots import _plot,_lplot,_scatter
-from ..util.units import _convert_length
+from ..util.units import _convert_length,_convert_time,_convert_velocity
 
 import matplotlib.pyplot as plt
 
+try:
+    import amuse.units.units as u
+except:
+    pass
+
 def find_centre(
     cluster,
     xstart=0.0,
@@ -65,7 +70,6 @@ def find_centre(
     nmax=100,
     method='harfst',
     nneighbour=6,
-    return_rhos=False,
 ):
     """Find the cluster's centre
 
@@ -100,8 +104,6 @@ def find_centre(
     if method=='casertano'
         nneighbour : int
             number of neighbours for calculation local densities
-        return_rhos : bool
-            return local densities (default: False)
 
     Returns
     -------
@@ -111,6 +113,13 @@ def find_centre(
     -------
     2019 - Written - Webb (UofT)
     """
+
+    cluster.save_cluster()
+    units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
+
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if indx is None:
         indx = np.ones(cluster.ntot, bool)
     elif np.sum(indx) == 0.0:
@@ -120,42 +129,22 @@ def find_centre(
     rhos=np.zeros(cluster.ntot)
 
     if density:
-        if return_rhos:
-            xc,yc,zc,vxc,vyc,vzc,rhos=find_centre_of_density(
-                    cluster=cluster,
-                    xstart=xstart,
-                    ystart=ystart,
-                    zstart=zstart,
-                    vxstart=vxstart,
-                    vystart=vystart,
-                    vzstart=vzstart,
-                    indx=indx,
-                    nsphere=nsphere,
-                    rmin=rmin,
-                    rmax=rmax,
-                    nmax=nmax,
-                    method=method,
-                    nneighbour=nneighbour,
-                    return_rhos=return_rhos,
-                )
-        else:
-            xc,yc,zc,vxc,vyc,vzc=find_centre_of_density(
-                cluster=cluster,
-                xstart=xstart,
-                ystart=ystart,
-                zstart=zstart,
-                vxstart=vxstart,
-                vystart=vystart,
-                vzstart=vzstart,
-                indx=indx,
-                nsphere=nsphere,
-                rmin=rmin,
-                rmax=rmax,
-                nmax=nmax,
-                method=method,
-                nneighbour=nneighbour,
-                return_rhos=return_rhos,
-            )
+        xc,yc,zc,vxc,vyc,vzc=find_centre_of_density(
+            cluster=cluster,
+            xstart=xstart,
+            ystart=ystart,
+            zstart=zstart,
+            vxstart=vxstart,
+            vystart=vystart,
+            vzstart=vzstart,
+            indx=indx,
+            nsphere=nsphere,
+            rmin=rmin,
+            rmax=rmax,
+            nmax=nmax,
+            method=method,
+            nneighbour=nneighbour,
+        )
     else:
 
         x = cluster.x[indx] - xstart
@@ -188,8 +177,10 @@ def find_centre(
         vyc = np.sum(cluster.m[indx] * cluster.vy[indx]) / np.sum(cluster.m[indx])
         vzc = np.sum(cluster.m[indx] * cluster.vz[indx]) / np.sum(cluster.m[indx])
 
-    if return_rhos:
-        return xc, yc, zc, vxc, vyc, vzc, rhos
+    cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
+
+    if cluster.units =='amuse':
+        return xc | u.pc, yc | u.pc, zc | u.pc, vxc | u.kms, vyc | u.kms, vzc | u.kms
     else:
         return xc, yc, zc, vxc, vyc, vzc
 
@@ -208,7 +199,6 @@ def find_centre_of_density(
     nmax=100,
     method='harfst',
     nneighbour=6,
-    return_rhos=False,
 ):
     """Find cluster's centre of density
 
@@ -241,8 +231,6 @@ def find_centre_of_density(
     if method=='casertano'
         nneighbour : int
             number of neighbours for calculation local densities
-        return_rhos : bool
-            return local densities (default: False)
 
     Returns
     -------
@@ -255,25 +243,28 @@ def find_centre_of_density(
     2022 - Written - Webb (UofT) - add method=='casertano'
     """
 
+    cluster.save_cluster()
+    units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if method=='harfst':
         xdc, ydc, zdc,vxdc, vydc, vzdc=find_centre_of_density_harfst(cluster,xstart=xstart,
             ystart=ystart,zstart=zstart,vxstart=vxstart,vystart=vystart,vzstart=vzstart,indx=indx,
             nsphere=nsphere,rmin=rmin,rmax=rmax,nmax=nmax)
     elif method=='casertano':
-        if return_rhos:
-            xdc, ydc, zdc,vxdc, vydc, vzdc, rhos=find_centre_of_density_casertano(cluster,nneighbour=nneighbour,return_rhos=return_rhos)
-        else:
-            xdc, ydc, zdc,vxdc, vydc, vzdc=find_centre_of_density_casertano(cluster,nneighbour=nneighbour,return_rhos=return_rhos)
+        xdc, ydc, zdc,vxdc, vydc, vzdc=find_centre_of_density_casertano(cluster,nneighbour=nneighbour)
 
-    if method=='casertano' and return_rhos:
-        return xdc, ydc, zdc,vxdc, vydc, vzdc, rhos
+    cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
+
+    if cluster.units =='amuse':
+        return xdc | u.pc, ydc | u.pc, zdc | u.pc, vxdc | u.kms, vydc | u.kms, vzdc | u.kms
     else:
         return xdc, ydc, zdc,vxdc, vydc, vzdc
 
 def find_centre_of_density_casertano(
     cluster,
     nneighbour=6,
-    return_rhos=False
 ):
 
     """Find cluster's centre of density
@@ -287,18 +278,12 @@ def find_centre_of_density_casertano(
         StarCluster
     nneighbour : int
         number of neighbours for calculation local densities
-    return_rhos : bool
-        return local densities (default: False)
 
     Returns
     -------
     xc,yc,zc,vxc,vyc,vzc : float
         coordinates of centre of mass
 
-    if return_rhos:
-        rhos : float
-            local density about each star
-
     HISTORY
     -------
     2022 - Written - Webb (UofT)
@@ -324,10 +309,7 @@ def find_centre_of_density_casertano(
     vydc=np.sum(rhos*cluster.vy/rhototal)
     vzdc=np.sum(rhos*cluster.vz/rhototal)
 
-    if return_rhos:
-        return xdc, ydc, zdc,vxdc, vydc, vzdc, rhos
-    else:
-        return xdc, ydc, zdc,vxdc, vydc, vzdc
+    return xdc, ydc, zdc,vxdc, vydc, vzdc
 
 
 def find_centre_of_density_harfst(
@@ -491,7 +473,7 @@ def relaxation_time(cluster, rad=None, coulomb=0.4, projected=False,method='spit
     Returns
     -------
        trelax : float
-          relaxation time within radius rad (in Myr)
+          relaxation time within radius rad
 
     History
     -------
@@ -540,6 +522,8 @@ def relaxation_time(cluster, rad=None, coulomb=0.4, projected=False,method='spit
 
     cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
 
+    trelax=_convert_time(trelax,'pckms',cluster)
+
     return trelax
 
 def half_mass_relaxation_time(cluster, coulomb=0.4, projected=False):
@@ -596,6 +580,9 @@ def half_mass_relaxation_time(cluster, coulomb=0.4, projected=False):
 
     cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
 
+    trh=_convert_time(trh,'pckms',cluster)
+
+
     return trh
 
 
@@ -652,10 +639,13 @@ def core_relaxation_time(cluster, coulomb=0.4, projected=False):
 
     cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
 
+    trc=_convert_time(trc,'pckms',cluster)
+
+
     return trc
 
 
-def energies(cluster, specific=True, ids=None, full=True, projected=False, parallel=False, **kwargs):
+def energies(cluster, specific=True, ids=None, projected=False, softening=0.0, full=True, parallel=False, **kwargs):
     """Calculate kinetic and potential energy of every star
     Parameters
     ----------
@@ -666,6 +656,10 @@ def energies(cluster, specific=True, ids=None, full=True, projected=False, paral
     ids: bool or int
       if given, find the energues of a subset of stars defined either by an array of
       star ids, or a boolean array that can be used to slice the cluster. (default: None)
+    projected : bool
+      use projected values (default: False)
+    softening : float
+      Plummer softening length in cluster.units (default: 0.0)
     full : bool
       calculate distance of full array of stars at once with numbra (default: True)
     parallel : bool
@@ -690,6 +684,8 @@ def energies(cluster, specific=True, ids=None, full=True, projected=False, paral
 
     if cluster.origin0 != 'cluster' and cluster.origin0 != 'centre':
         cluster.to_cluster(sortstars=False)
+    if cluster.units=='amuse':
+        cluster.to_pckms()
 
     grav=_get_grav(cluster)
 
@@ -723,9 +719,9 @@ def energies(cluster, specific=True, ids=None, full=True, projected=False, paral
                                  cluster.z[ids], cluster.m[ids]]).T
 
         if parallel:
-            pot = grav * np.array(_potential_energy_subset_parallel(cluster_sub, cluster_full))
+            pot = grav * np.array(_potential_energy_subset_parallel(cluster_sub, cluster_full,softening))
         else:
-            pot = grav * np.array(_potential_energy_subset(cluster_sub, cluster_full))
+            pot = grav * np.array(_potential_energy_subset(cluster_sub, cluster_full,softening))
 
         if specific:
             pot /= cluster.m[ids]
@@ -737,9 +733,9 @@ def energies(cluster, specific=True, ids=None, full=True, projected=False, paral
         else:
             x = np.array([cluster.x, cluster.y, cluster.z, cluster.m]).T
         if parallel:
-            pot = grav * np.array(_potential_energy_parallel(x))
+            pot = grav * np.array(_potential_energy_parallel(x,softening))
         else:
-            pot = grav * np.array(_potential_energy(x))
+            pot = grav * np.array(_potential_energy(x,softening))
 
         if specific:
             pot /= cluster.m
@@ -756,9 +752,9 @@ def energies(cluster, specific=True, ids=None, full=True, projected=False, paral
                 m = cluter.m[i] * cluster.m
 
             if projected:
-                dr = np.sqrt(dx ** 2.0 + dy ** 2.0)
+                dr = np.sqrt(dx ** 2.0 + dy ** 2.0 + softening**2.) 
             else:
-                dr = np.sqrt(dx ** 2.0 + dy ** 2.0 + dz ** 2.0)
+                dr = np.sqrt(dx ** 2.0 + dy ** 2.0 + dz ** 2.0 + softening**2.)
 
             indx = dr != 0.0
             gmr = -grav * m[indx] / dr[indx]
@@ -767,11 +763,17 @@ def energies(cluster, specific=True, ids=None, full=True, projected=False, paral
 
     cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
 
+    if cluster.units=='amuse' and specific:
+        kin=kin | (u.kms*u.kms)
+        pot=pot | (u.kms*u.kms)
+    elif cluster.units=='amuse' and not specific:
+        kin=kin | (u.MSun*u.kms*u.kms)
+        pot=pot | (u.MSun*u.kms*u.kms)
     return kin, pot
 
 
 @numba.njit
-def _potential_energy(cluster):
+def _potential_energy(cluster,softening=0.0):
     """Find potential energy for each star in a cluster
     - uses numba
 
@@ -779,7 +781,8 @@ def _potential_energy(cluster):
     ----------
     cluster : float
         positions and masses of stars within the StarCluster
-
+    softening : float
+      Plummer softening length in cluster.units (default: 0.0)
     Returns
     -------
         pot : float
@@ -797,14 +800,14 @@ def _potential_energy(cluster):
             if r==0: r=np.nan
 
             m2 = cluster[i, 3] * cluster[j, 3]
-            pot[i] += -m2 / r
-            pot[j] += -m2 / r
+            pot[i] += -m2 / np.sqrt(r**2.+softening**2.)
+            pot[j] += -m2 / np.sqrt(r**2.+softening**2.)
 
     return pot
 
 
 @numba.njit(parallel=True)
-def _potential_energy_parallel(cluster):
+def _potential_energy_parallel(cluster,softening=0.0):
     """Find potential energy for each star in a cluster in parallel
     - uses numba
 
@@ -812,7 +815,8 @@ def _potential_energy_parallel(cluster):
     ----------
     cluster : class
         positions and masses of stars within the StarCluster
-
+    softening : float
+      Plummer softening length in cluster.units (default: 0.0)
     Returns
     -------
         pot : float
@@ -830,14 +834,14 @@ def _potential_energy_parallel(cluster):
             if r==0: r=np.nan
 
             m2 = cluster[i, 3] * cluster[j, 3]
-            pot[i] += -m2 / r
-            pot[j] += -m2 / r
+            pot[i] += -m2 / np.sqrt(r**2.+softening**2.)
+            pot[j] += -m2 / np.sqrt(r**2.+softening**2.)
 
     return pot
 
 
 @numba.njit()
-def _potential_energy_subset(cluster_sub, cluster_full):
+def _potential_energy_subset(cluster_sub, cluster_full,softening=0.0):
     """Find the potential energy for a subset of stars in a bigger cluster
     
     Parameters
@@ -846,7 +850,8 @@ def _potential_energy_subset(cluster_sub, cluster_full):
         2d numpy array with x, y, z and mass at each index of the sub cluster
     cluster_full : float
         2d numpy array with x, y, z and mass at each index of the cluster
-    
+    softening : float
+      Plummer softening length in cluster.units (default: 0.0)
     Returns:
     --------
         potential : numpy array
@@ -866,12 +871,13 @@ def _potential_energy_subset(cluster_sub, cluster_full):
 
                 m = cluster_sub[i,3] * cluster_full[j,3]
 
-                pot[i] += -m / r
+                pot[i] += -m / np.sqrt(r**2.+softening**2.)
 
         
     return pot
+    
 @numba.njit()
-def _potential_energy_subset_parallel(cluster_sub, cluster_full):
+def _potential_energy_subset_parallel(cluster_sub, cluster_full,softening=0.0):
     """Find the potential energy for a subset of stars in a bigger cluster
     
     Parameters
@@ -880,7 +886,8 @@ def _potential_energy_subset_parallel(cluster_sub, cluster_full):
         2d numpy array with x, y, z and mass at each index of the sub cluster
     cluster_full : float
         2d numpy array with x, y, z and mass at each index of the cluster
-    
+    softening : float
+      Plummer softening length in cluster.units (default: 0.0)    
     Returns:
     --------
         potential : numpy array
@@ -900,12 +907,12 @@ def _potential_energy_subset_parallel(cluster_sub, cluster_full):
 
                 m = cluster_sub[i,3] * cluster_full[j,3]
 
-                pot[i] += -m / r
+                pot[i] += -m / np.sqrt(r**2.+softening**2.)
         
     return pot
 
 
-def closest_star(cluster, projected=False):
+def closest_star(cluster, projected=False, argument=False):
     """Find distance to closest star for each star
     - uses numba
 
@@ -913,26 +920,37 @@ def closest_star(cluster, projected=False):
     ----------
     cluster : class
         positions of stars within the StarCluster
+    projected : bool
+      use projected values (default: False)
+    argument : bool
+      return argument of closest star as well (default: False)
 
     Returns
     -------
         minimum_distance : float
             distance to closest star for each star
 
+        if argument:
+            arg : int
+                argument of closest star for each star            
+
     History
     -------
        2019 - Written - Webb (UofT)
     """
 
     if projected:
-        z = np.zeros(cluster.ntot)
-        x = np.array([cluster.x, cluster.y, z]).T
+        x=np.column_stack([cluster.x,cluster.y])
     else:
-        x = np.array([cluster.x, cluster.y, cluster.z]).T
-    return minimum_distance(x)
-
+        x=np.column_stack([cluster.x,cluster.y,cluster.z])
 
+    tree=cKDTree(x)
+    dist, arg = tree.query(x, k=2)
 
+    if argument:
+        return dist[:,1],arg[:,1]
+    else:
+        return dist[:,1]
 
 def rlagrange(cluster, nlagrange=10, mfrac=None, projected=False):
     """Calculate lagrange radii of the cluster by mass
@@ -1361,6 +1379,12 @@ def mass_function(
     2018 - Written - Webb (UofT)
     """
 
+    cluster.save_cluster()
+    units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
+
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if projected:
         r = cluster.rpro
         v = cluster.vpro
@@ -1480,6 +1504,9 @@ def mass_function(
                 -1000.0,
             )
 
+    cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
+
+
 def tapered_mass_function(
     cluster,
     mmin=None,
@@ -1558,6 +1585,12 @@ def tapered_mass_function(
     2018 - Written - Webb (UofT)
     """
 
+    cluster.save_cluster()
+    units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
+
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if projected:
         r = cluster.rpro
         v = cluster.vpro
@@ -1661,6 +1694,7 @@ def tapered_mass_function(
             -1000.0,
         )
 
+    cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
 
 def tpl_func(m,A,alpha,mc,beta):
 
@@ -1743,6 +1777,12 @@ def eta_function(
     2018 - Written - Webb (UofT)
     """
 
+    cluster.save_cluster()
+    units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
+
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if projected:
         r = cluster.rpro
         v = cluster.vpro
@@ -1826,8 +1866,12 @@ def eta_function(
             if filename != None:
                 plt.savefig(filename)
 
+        cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
+
         return m_mean, sigvm, eta, eeta, yeta, eyeta
     else:
+        cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
+
         print("NOT ENOUGH STARS TO ESTIMATE SIGMA-MASS RELATION")
         return (
             np.zeros(nmass),
@@ -1932,6 +1976,11 @@ def meq_function(
     -------
     2020
     """
+    cluster.save_cluster()
+    units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
+
+    if cluster.units=='amuse':
+        cluster.to_pckms()
 
     m_mean, sigvm, meq, emq, sigma0, esigma0 = eta_function(cluster,
             mmin=mmin,
@@ -1953,6 +2002,8 @@ def meq_function(
             **kwargs
         )
 
+    cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
+
     return m_mean, sigvm, meq, emq, sigma0, esigma0
 
 def ckin(
@@ -2015,6 +2066,11 @@ def ckin(
     -------
     2020
     """
+    cluster.save_cluster()
+    units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
+
+    if cluster.units=='amuse':
+        cluster.to_pckms()
 
     rn=rlagrange(cluster, nlagrange=10, projected=projected)
 
@@ -2060,6 +2116,9 @@ def ckin(
 
     ck=meq/meq50
 
+    cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
+
+
     return ck
 
 def rcore(
@@ -2110,6 +2169,9 @@ def rcore(
     cluster.save_cluster()
     units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
 
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if cluster.origin0 != 'centre':
         if cluster.xc==0. and cluster.yc==0. and cluster.zc==0.0:
             if method=='casertano':
@@ -2142,10 +2204,11 @@ def rcore(
         rc2=np.sum((rhos**2.)*(r**2.))/np.sum((rhos**2.))
         rc=np.sqrt(rc2)
 
-        if plot: 
+        if plot:
+            nrad=int(np.ceil(1./mfrac)) 
             rprof,pprof,nprof=_rho_prof(cluster,nrad=nrad,projected=projected,plot=False,**kwargs)
             rcindx=(cluster.r<rc)
-            mc=np.sum(cluster.mtot[rcindx])
+            mc=np.sum(cluster.m[rcindx])
             volc=(4./3.)*np.pi*(rc**3.)
             rho_c=mc/volc
 
@@ -2356,6 +2419,9 @@ def rtidal(
     cluster.save_cluster()
     units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
 
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     ro,vo,zo,solarmotion=cluster._ro,cluster._vo,cluster._zo,cluster._solarmotion
 
     if cluster.origin0 != 'cluster' and cluster.origin0 != 'centre':
@@ -2529,6 +2595,9 @@ def rlimiting(
     cluster.save_cluster()
     units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
 
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     ro,vo,zo,solarmotion=cluster._ro,cluster._vo,cluster._zo,cluster._solarmotion
 
     mo=conversion.mass_in_msol(ro=ro,vo=vo)
@@ -2748,6 +2817,9 @@ def _rho_prof(
     cluster.save_cluster()
     units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
 
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if cluster.origin0 != 'cluster' and cluster.origin0 != 'centre':
         cluster.to_centre(sortstars=normalize)
     elif normalize:
