@@ -27,6 +27,9 @@ from ..util.constants import _get_grav
 from ..util.plots import _lplot,_plot
 from ..util.coordinates import sphere_coords
 from .functions import mass_function, eta_function
+from ..util.units import _convert_length,_convert_time,_convert_velocity,_convert_density,_convert_mass,_convert_square_velocity
+
+
 
 import matplotlib.pyplot as plt
 
@@ -107,6 +110,9 @@ def rho_prof(
     cluster.save_cluster()
     units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
 
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if cluster.origin0 != 'cluster' and cluster.origin0 != 'centre':
         cluster.to_centre(sortstars=normalize)
     elif normalize:
@@ -234,8 +240,13 @@ def rho_prof(
         if filename != None:
             plt.savefig(filename)
 
+
+    units=cluster.units
     cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
 
+    rprof=_convert_length(rprof,units,cluster)
+    pprof=_convert_density(pprof,units,cluster,projected=projected)
+
     if normalize:
         rprof/=cluster.rm
 
@@ -318,6 +329,9 @@ def m_prof(
     cluster.save_cluster()
     units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
 
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if cluster.origin0 != 'cluster' and cluster.origin0 != 'centre':
         cluster.to_centre(sortstars=normalize)
     elif normalize:
@@ -429,8 +443,12 @@ def m_prof(
         if filename != None:
             plt.savefig(filename)
 
+    units=cluster.units
     cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
 
+    rprof=_convert_length(rprof,units,cluster)
+    mprof=_convert_mass(mprof,units,cluster)
+
     if normalize:
         rprof/=cluster.rm
 
@@ -536,6 +554,9 @@ def alpha_prof(
     cluster.save_cluster()
     units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
 
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if cluster.origin0 != 'cluster' and cluster.origin0 != 'centre':
         cluster.to_centre(sortstars=normalize)
     elif normalize:
@@ -644,7 +665,9 @@ def alpha_prof(
         if filename != None:
             plt.savefig(filename)
 
+    units=cluster.units
     cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
+    rprofn=_convert_length(rprofn,units,cluster)
 
 
     if aerror:
@@ -728,6 +751,9 @@ def sigv_prof(
     cluster.save_cluster()
     units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
 
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if cluster.origin0 != 'cluster' and cluster.origin0 != 'centre':
         cluster.to_centre(sortstars=normalize)
     elif normalize:
@@ -846,6 +872,7 @@ def sigv_prof(
 
             sigvprof[i]=sigv
 
+    units=cluster.units
     cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
 
 
@@ -870,6 +897,9 @@ def sigv_prof(
         if filename != None:
             plt.savefig(filename)
 
+    rprofn=_convert_length(rprofn,units,cluster)
+    sigvprof=_convert_velocity(sigvprof,units,cluster)
+
     return rprofn, sigvprof
 
 def beta_prof(
@@ -943,6 +973,9 @@ def beta_prof(
     cluster.save_cluster()
     units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
 
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if cluster.origin0 != 'cluster' and cluster.origin0 != 'centre':
         cluster.to_centre(sortstars=normalize)
     elif normalize:
@@ -1032,6 +1065,7 @@ def beta_prof(
 
             betaprof[i]=beta
 
+    units=cluster.units
     cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
 
 
@@ -1056,6 +1090,8 @@ def beta_prof(
         if filename != None:
             plt.savefig(filename)
 
+    if not normalize: rprofn=_convert_length(rprofn,units,cluster)
+
     return rprofn, betaprof
 
 
@@ -1134,6 +1170,9 @@ def v_prof(
     cluster.save_cluster()
     units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
 
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if cluster.origin0 != 'cluster' and cluster.origin0 != 'centre':
         cluster.to_centre(sortstars=normalize)
     elif normalize:
@@ -1250,6 +1289,7 @@ def v_prof(
             vmean = np.mean(v[rindx])
             vprof[i]=vmean
 
+    units=cluster.units
     cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
 
 
@@ -1274,6 +1314,9 @@ def v_prof(
         if filename != None:
             plt.savefig(filename)
 
+    if not normalize: rprofn=_convert_length(rprofn,units,cluster)
+    vprof=_convert_velocity(vprof,units,cluster)
+
     return rprofn, vprof
 
 
@@ -1352,6 +1395,9 @@ def v2_prof(
     cluster.save_cluster()
     units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
 
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if cluster.origin0 != 'cluster' and cluster.origin0 != 'centre':
         cluster.to_centre(sortstars=normalize)
     elif normalize:
@@ -1458,6 +1504,7 @@ def v2_prof(
             vmean = np.mean(v[rindx]**2.)
             vprof[i]=vmean
 
+    units=cluster.units
     cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
 
 
@@ -1482,6 +1529,9 @@ def v2_prof(
         if filename != None:
             plt.savefig(filename)
 
+    if not normalize: rprofn=_convert_length(rprofn,units,cluster)
+    vprof=_convert_square_velocity(vprof,units,cluster)
+
     return rprofn, vprof
 
 def eta_prof(
@@ -1567,6 +1617,9 @@ def eta_prof(
     cluster.save_cluster()
     units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
 
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if cluster.origin0 != 'cluster' and cluster.origin0 != 'centre':
         cluster.to_centre(sortstars=normalize)
     elif normalize:
@@ -1708,8 +1761,10 @@ def eta_prof(
         if filename != None:
             plt.savefig(filename)
 
+    units=cluster.units
     cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
 
+    if not normalize: rprofn=_convert_length(rprofn,units,cluster)
 
     return rprofn, eprof, deta, edeta, ydeta, eydeta
 
@@ -1905,6 +1960,9 @@ def vcirc_prof(
     cluster.save_cluster()
     units0,origin0, rorder0, rorder_origin0 = cluster.units0,cluster.origin0, cluster.rorder0, cluster.rorder_origin0
 
+    if cluster.units=='amuse':
+        cluster.to_pckms()
+
     if cluster.origin0 != 'cluster' and cluster.origin0 != 'centre':
         cluster.to_centre(sortstars=normalize)
     else:
@@ -2044,7 +2102,13 @@ def vcirc_prof(
         if filename != None:
             plt.savefig(filename)
 
+    units=cluster.units
     cluster.return_cluster(units0,origin0, rorder0, rorder_origin0)
 
+    if not normalize: rprof=_convert_length(rprof,units,cluster)
+    vcprof=_convert_velocity(vcprof,units,cluster)
+
+    rvmax=_convert_length(rvmax,units,cluster)
+    vmax=_convert_velocity(vmax,units,cluster)
 
     return rprof, vcprof, rvmax, vmax
