@@ -98,7 +98,7 @@ def _get_nbody6pp(conf3, bev82=None, sev83=None, snap40=None, ofile=None, advanc
             cluster.add_nbody6(nbody6list)
 
         if binaries:
-            cluster.add_stars(xc1,xc2,xc3,vc1,vc2,vc3,mbtot,i_d1)
+            cluster.add_stars(xc1,xc2,xc3,vc1,vc2,vc3,mbtot,i_d1,analyze=False)
             pb=np.array(pb)
             semi=np.array(semi)
             m1b=np.array(m1b)
@@ -125,10 +125,10 @@ def _get_nbody6pp(conf3, bev82=None, sev83=None, snap40=None, ofile=None, advanc
             xb2, yb2, zb2 = bmr2 * xr1, bmr2 * xr2, bmr2 * xr3
             vxb2, vyb2, vzb2  = bmr2 * vr1,  bmr2 * vr2,  bmr2 * vr3
 
-            cluster.add_binary_stars(xb1, yb1, zb1, vxb1, vyb1, vzb1, xb2, yb2, zb2, vxb2, vyb2, vzb2)
+            cluster.add_binary_stars(xb1, yb1, zb1, vxb1, vyb1, vzb1, xb2, yb2, zb2, vxb2, vyb2, vzb2,set_com=False)
 
 
-        cluster.add_stars(x, y, z, vx, vy, vz, m, i_d)
+        cluster.add_stars(x, y, z, vx, vy, vz, m, i_d, analyze=False)
         cluster.add_sse(kw,lum,rs)
 
         if binaries: 
@@ -149,7 +149,7 @@ def _get_nbody6pp(conf3, bev82=None, sev83=None, snap40=None, ofile=None, advanc
             cluster.zc*=cluster.rbar
             cluster.tphys*=cluster.tbar
 
-        cluster.to_nbody()
+        cluster.to_nbody(analyze=False)
 
         if binaries: cluster.nb = len(semi)
 
@@ -157,7 +157,6 @@ def _get_nbody6pp(conf3, bev82=None, sev83=None, snap40=None, ofile=None, advanc
     else:
         ntot,alist,x,y,z,vx,vy,vz,m,i_d,rhos,xns,pot=_get_nbody6pp_conf3(conf3,nsnap=nsnap,**kwargs)
 
-
         cluster = StarCluster(
             alist[0],
             units="nbody",
@@ -172,32 +171,49 @@ def _get_nbody6pp(conf3, bev82=None, sev83=None, snap40=None, ofile=None, advanc
             cluster.add_nbody6(
             alist[13], alist[12], alist[2], alist[4], alist[6], alist[7], alist[8], alist[3], alist[11],alist[10],alist[17], ntot, alist[1], ntot+alist[1]
         )
-            cluster.add_stars(x, y, z, vx, vy, vz, m, i_d)
-            cluster.rhos=rhos
+            nb=int(alist[1])
+            cluster.add_stars(x, y, z, vx, vy, vz, m, i_d,nb=nb,analyze=False)
+            cluster.rhos=np.zeros(ntot-nb)
+            pots=np.zeros(ntot-nb)
 
-            v=np.sqrt(vx**2.+vy**2.+vz**2.)
-            ek=0.5*m*v**2.
-            cluster.add_energies(ek,pot)
+            if nb>0:
+                binargs=np.arange(0,nb,1)
+                binargs1=np.arange(0,int(2*nb),2)
+                binargs2=binargs1+1
+                cluster.rhos[binargs]=rhos[binargs1]+rhos[binargs2]
+                pots[binargs]=pot[binargs1]+pot[binargs2]
+
+            cluster.rhos[nb:]=rhos[2*nb:]
+            pots[nb:]=pot[2*nb:]
+
+            v=np.sqrt(cluster.vx**2.+cluster.vy**2.+cluster.vz**2.)
+            ek=0.5*cluster.m*v**2.
+            cluster.add_energies(ek,pots)
 
         if bev82 is not None and sev83 is not None:
+
+            kws=np.ones(ntot-nb)*-10.0
+            zl1s=np.ones(ntot-nb)*-10.0
+            r1s=np.ones(ntot-nb)*-10.0
+
             arg,i_d,kw,ri,m1,zl1,r1,te,i_d1,i_d2,kw1,kw2,kwb,rib,ecc,pb,semi,m1b,m2b,zl1b,zl2b,r1b,r2b,te1,te2=_get_nbody6pp_ev(bev82,sev83,nsnap=nsnap,**kwargs)
             #Convert from fortran array address to python
-            arg-=1
-
-            if len(kw) != len(x):
-                if verbose: print('SSE/BSE NBODY6++ ERROR',len(x)-len(kw))
-                kws=np.ones(len(x))*-10.0
-                zl1s=np.ones(len(x))*-10.0
-                r1s=np.ones(len(x))*-10.0
-
-                kws[arg]=kw
-                zl1s[arg]=zl1
-                r1s[arg]=r1
-                cluster.add_sse(kws,zl1s,r1s)
 
+            if nb>0:
+                kws[binargs]=np.maximum(kw1,kw2)
+                zl1s[binargs]=zl1b+zl2b
+                r1s[binargs]=r1b+r2b
+                args=arg[2*nb:]-nb-1
             else:
+                args=arg-1
 
-                cluster.add_sse(kw,zl1,r1)
+            args=args.astype(int)
+
+            kws[args]=kw[2*nb:]
+            zl1s[args]=zl1[2*nb:]
+            r1s[args]=r1[2*nb:]
+
+            cluster.add_sse(kws,zl1s,r1s)
 
             pb=(10.0**pb)/cluster.tbar_days
             semi=(10.0**semi)/cluster.rbar_su
@@ -206,6 +222,10 @@ def _get_nbody6pp(conf3, bev82=None, sev83=None, snap40=None, ofile=None, advanc
 
             cluster.add_bse(i_d1,i_d2,kw1,kw2,kwb,ecc,pb,semi,m1b,m2b,zl1b,zl2b,r1b,r2b)
 
+
+    if np.sum(cluster.kw<0) >0:
+        if verbose: print('SSE/BSE NBODY6++ ERROR',np.sum(cluster.kw<0))
+
     if kwargs.get("analyze", True) and cluster.ntot>0:
         sortstars=kwargs.get("sortstars", True)
         cluster.analyze(sortstars=sortstars)
@@ -313,74 +333,6 @@ def _get_nbody6pp_ev(bev, sev, **kwargs):
     header=bev.readline().split()
     nb,tphys=int(header[0]),float(header[1])
 
-    """
-    i_d1=np.array([])
-    i_d2=np.array([])
-    kw1=np.array([])
-    kw2=np.array([])
-    kwb=np.array([])
-    rib=np.array([])
-    ecc=np.array([])
-    pb=np.array([])
-    semi=np.array([])
-    m1b=np.array([])
-    m2b=np.array([])
-    zl1b=np.array([])
-    zl2b=np.array([])
-    r1b=np.array([])
-    r2b=np.array([])
-    te1=np.array([])
-    te2=np.array([])
-
-    if nb>0:
-
-        for i in range(0,nb):
-            data=bev.readline().split()
-            if len(data)==0:
-                if verbose: print('Missing stars in BEV Star')
-                break
-            arg1=int(data[1])
-            arg2=int(data[2])
-            i_d1=np.append(i_d1,int(data[3]))
-            i_d2=np.append(i_d2,int(data[4]))
-            kw1=np.append(kw1,int(data[5]))
-            kw2=np.append(kw2,int(data[6]))
-            kwb=np.append(kwb,int(data[7]))
-            rib=np.append(rib,float(data[8]))
-            ecc=np.append(ecc,float(data[9]))
-            pb=np.append(pb,float(data[10]))
-            semi=np.append(semi,float(data[11]))
-            m1b=np.append(m1b,float(data[12]))
-            m2b=np.append(m2b,float(data[13]))
-
-            if data[14]=='NaN':
-                zl1b=np.append(zl1b,0.)
-                zl2b=np.append(zl2b,0.)
-                r1b=np.append(r1b,0.)
-                r2b=np.append(r2b,0.)
-                te1=np.append(te1,0.)
-                te2=np.append(te2,0.)
-            else:
-                zl1b=np.append(zl1b,float(data[14]))
-                zl2b=np.append(zl2b,float(data[15]))
-                r1b=np.append(r1b,float(data[16]))
-                r2b=np.append(r2b,float(data[17]))
-                te1=np.append(te1,float(data[18]))
-                te2=np.append(te2,float(data[19]))
-
-            #Add select parameters to single star array
-            arg=np.append(arg,arg1)
-            arg=np.append(arg,arg2)
-            i_d=np.append(i_d,i_d1[-1])
-            i_d=np.append(i_d,i_d2[-1])
-            kw=np.append(kw,kw1[-1])
-            kw=np.append(kw,kw2[-1])
-            zl1=np.append(zl1,zl1b[-1])
-            zl1=np.append(zl1,zl2b[-1])
-            r1=np.append(r1,r1b[-1])
-            r1=np.append(r1,r2b[-1])
-    """
-
     if nb>0:
 
         data=np.loadtxt(bev.name,skiprows=1)
@@ -466,31 +418,6 @@ def _get_nbody6pp_ev(bev, sev, **kwargs):
     header=sev.readline().split()
     ntot,tphys=int(header[0]),float(header[1])
 
-    """
-
-    for i in range(0,ntot):
-        data=sev.readline().split()
-
-        if len(data)==0:
-            if verbose: print('Missing stars in SEV Star',i,ntot)
-            break
-
-        arg=np.append(arg,int(data[1]))
-        i_d=np.append(i_d,int(data[2]))
-        kw=np.append(kw,int(data[3]))
-        ri=np.append(ri,float(data[4]))
-        m1=np.append(m1,float(data[5]))
-
-        if data[6]=='NaN':
-            zl1=np.append(zl1,0.)
-            r1=np.append(r1,0.)
-            te=np.append(te,0.)
-        else:
-            zl1=np.append(zl1,float(data[6]))
-            r1=np.append(r1,float(data[7]))
-            te=np.append(te,float(data[8]))
-    """
-
     data=np.loadtxt(sev.name,skiprows=1)
 
     arg=data[:,1].astype(int)
